From 30d45de87473d683961fda9aee1ce2068f3b2e7f Mon Sep 17 00:00:00 2001
From: Lukas Durfina <lukas.durfina@gmail.com>
Date: Fri, 29 Aug 2008 16:22:36 +0200
Subject: [PATCH] libVLC: add new functions for phonon backend.
 There are functions for audio/video effects, changing audio devices,
 better dvd, subtitles and audio tracks handling.

---
 include/vlc/libvlc.h                             |  496 ++++++++++++++++++++
 include/vlc/libvlc_events.h                      |    5 +
 include/vlc/libvlc_structures.h                  |   64 +++
 modules/gui/qt4/components/interface_widgets.cpp |    1 +
 modules/video_filter/colorthres.c                |    2 +
 src/control/audio.c                              |  356 ++++++++++++++
 src/control/event.c                              |    1 +
 src/control/libvlc_internal.h                    |   47 ++-
 src/control/media_player.c                       |  425 +++++++++++++++++-
 src/control/video.c                              |  542 ++++++++++++++++++++++
 src/libvlc.c                                     |    4 +
 src/libvlc.sym                                   |   39 ++
 src/video_output/video_output.c                  |    4 +
 13 files changed, 1980 insertions(+), 6 deletions(-)

diff --git a/include/vlc/libvlc.h b/include/vlc/libvlc.h
index 7f2f384..3f839d1 100644
--- a/include/vlc/libvlc.h
+++ b/include/vlc/libvlc.h
@@ -586,6 +586,7 @@ VLC_PUBLIC_API void libvlc_media_player_set_position( libvlc_media_player_t *, f
  */
 VLC_PUBLIC_API void libvlc_media_player_set_chapter( libvlc_media_player_t *, int, libvlc_exception_t *);
 
+
 /**
  * Get movie chapter
  *
@@ -606,6 +607,60 @@ VLC_PUBLIC_API int libvlc_media_player_get_chapter_count( libvlc_media_player_t
 VLC_PUBLIC_API int libvlc_media_player_will_play        ( libvlc_media_player_t *, libvlc_exception_t *);
 
 /**
+ * Get title chapter count
+ *
+ * \param p_mi the Media Player
+ * \param i_title title
+ * \param p_e an initialized exception pointer
+ * \return number of chapters in title
+ */
+VLC_PUBLIC_API int libvlc_media_player_get_chapter_count_for_title(
+                       libvlc_media_player_t *, int, libvlc_exception_t *);
+
+/**
+ * Set movie title
+ *
+ * \param p_mi the Media Player
+ * \param i_title title number to play
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_media_player_set_title( libvlc_media_player_t *, int, libvlc_exception_t *);
+
+/**
+ * Get movie title
+ *
+ * \param p_mi the Media Player
+ * \param p_e an initialized exception pointer
+ * \return title number currently playing
+ */
+VLC_PUBLIC_API int libvlc_media_player_get_title( libvlc_media_player_t *, libvlc_exception_t *);
+
+/**
+ * Get movie title count
+ *
+ * \param p_mi the Media Player
+ * \param p_e an initialized exception pointer
+ * \return title number count
+ */
+VLC_PUBLIC_API int libvlc_media_player_get_title_count( libvlc_media_player_t *, libvlc_exception_t *);
+
+/**
+ * Set previous chapter
+ *
+ * \param p_mi the Media Player
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_media_player_previous_chapter( libvlc_media_player_t *, libvlc_exception_t *);
+
+/**
+ * Set next chapter
+ *
+ * \param p_mi the Media Player
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_media_player_next_chapter( libvlc_media_player_t *, libvlc_exception_t *);
+
+/**
  * Get movie play rate
  *
  * \param p_mi the Media Player
@@ -667,6 +722,20 @@ VLC_PUBLIC_API int libvlc_media_player_is_seekable( libvlc_media_player_t *p_mi,
  */
 VLC_PUBLIC_API int libvlc_media_player_can_pause( libvlc_media_player_t *p_mi, libvlc_exception_t *p_e );
 
+/**
+ * Release (free) libvlc_track_description_t
+ *
+ * \param p_track_description the structure to release
+ */
+VLC_PUBLIC_API void libvlc_track_description_release( libvlc_track_description_t *p_track_description );
+
+/**
+ * Release (free) libvlc_filter_parameter_list_t
+ *
+ * \param p_parameter_list the linked list to release
+ */
+VLC_PUBLIC_API void libvlc_filter_parameters_release( libvlc_filter_parameter_list_t *p_parameter_list );
+
 /** \defgroup libvlc_video libvlc_video
  * \ingroup libvlc_media_player
  * LibVLC Video handling
@@ -745,6 +814,25 @@ VLC_PUBLIC_API void libvlc_video_set_aspect_ratio( libvlc_media_player_t *, char
 VLC_PUBLIC_API int libvlc_video_get_spu( libvlc_media_player_t *, libvlc_exception_t * );
 
 /**
+ * Get the number of available video subtitles.
+ *
+ * \param p_mediaplayer the media player
+ * \param p_e an initialized exception pointer
+ * \return the number of available video subtitles
+ */
+VLC_PUBLIC_API int libvlc_video_get_spu_count( libvlc_media_player_t *, libvlc_exception_t * );
+
+/**
+ * Get the description of available video subtitles.
+ *
+ * \param p_mediaplayer the media player
+ * \param p_e an initialized exception pointer
+ * \return list containing description of available video subtitles
+ */
+VLC_PUBLIC_API libvlc_track_description_t *
+        libvlc_video_get_spu_description( libvlc_media_player_t *, libvlc_exception_t * );
+
+/**
  * Set new video subtitle.
  *
  * \param p_mediaplayer the media player
@@ -764,6 +852,27 @@ VLC_PUBLIC_API void libvlc_video_set_spu( libvlc_media_player_t *, int , libvlc_
 VLC_PUBLIC_API int libvlc_video_set_subtitle_file( libvlc_media_player_t *, char *, libvlc_exception_t * );
 
 /**
+ * Get the description of available titles.
+ *
+ * \param p_mediaplayer the media player
+ * \param p_e an initialized exception pointer
+ * \return list containing description of available titles
+ */
+VLC_PUBLIC_API libvlc_track_description_t *
+        libvlc_video_get_title_description( libvlc_media_player_t *, libvlc_exception_t * );
+
+/**
+ * Get the description of available chapters for specific title.
+ *
+ * \param p_mediaplayer the media player
+ * \param i_title selected title
+ * \param p_e an initialized exception pointer
+ * \return list containing description of available chapter for title i_title
+ */
+VLC_PUBLIC_API libvlc_track_description_t *
+        libvlc_video_get_chapter_description( libvlc_media_player_t *, int, libvlc_exception_t * );
+
+/**
  * Get current crop filter geometry.
  *
  * \param p_mediaplayer the media player
@@ -808,6 +917,43 @@ VLC_PUBLIC_API int libvlc_video_get_teletext( libvlc_media_player_t *, libvlc_ex
 VLC_PUBLIC_API void libvlc_video_set_teletext( libvlc_media_player_t *, int, libvlc_exception_t * );
 
 /**
+ * Get number of available video tracks.
+ *
+ * \param p_mi media player
+ * \param p_e an initialized exception
+ * \return the number of available video tracks (int)
+ */
+VLC_PUBLIC_API int libvlc_video_get_track_count( libvlc_media_player_t *,  libvlc_exception_t * );
+
+/**
+ * Get the description of available video tracks.
+ *
+ * \param p_mi media player
+ * \param p_e an initialized exception
+ * \return list with description of available video tracks
+ */
+VLC_PUBLIC_API libvlc_track_description_t *
+        libvlc_video_get_track_description( libvlc_media_player_t *,  libvlc_exception_t * );
+
+/**
+ * Get current video track.
+ *
+ * \param p_mi media player
+ * \param p_e an initialized exception pointer
+ * \return the video track (int)
+ */
+VLC_PUBLIC_API int libvlc_video_get_track( libvlc_media_player_t *, libvlc_exception_t * );
+
+/**
+ * Set video track.
+ *
+ * \param p_mi media player
+ * \param i_track the track (int)
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_video_set_track( libvlc_media_player_t *, int, libvlc_exception_t * );
+
+/**
  * Take a snapshot of the current video window.
  *
  * If i_width AND i_height is 0, original size is used.
@@ -876,6 +1022,150 @@ VLC_PUBLIC_API void libvlc_video_set_size( libvlc_instance_t *, int, int, libvlc
  */
 VLC_PUBLIC_API void libvlc_video_set_viewport( libvlc_instance_t *, const libvlc_rectangle_t *, const libvlc_rectangle_t *, libvlc_exception_t * );
 
+/** \defgroup libvlc_video_filter libvlc_video_filter
+ * \ingroup libvlc_video
+ * LibVLC Video filter handling
+ * @{
+ */
+
+ /**
+  * Video filter names
+  *
+  * They are for: libvlc_video_filter_add / remove
+  */
+ typedef enum libvlc_video_filter_names_t {
+     ADJUST,          /**< Adjust filter - brightness, contrast, gamma, hue, saturation */
+     ATMOLIGHT,
+     BLUESCREEN,
+     COLORTHRESHOLD,
+     DEINTERLACE,
+     ERASE,
+     EXTRACT,
+     GAUSSIAN_BLUR,
+     GRADIENT,
+     GRAIN,
+     INVERT,
+     MOTIONBLUR,
+     MOTIONDETECT,
+     NOISE,
+     POSTPROCESS,
+     PSYCHEDELIC,
+     RIPPLE,
+     ROTATE,
+     SHARPEN,
+     WAVE
+ } libvlc_video_filter_names_t;
+
+/**
+ * Add video filter
+ *
+ * \param p_mediaplayer the media player
+ * \param filter video filter @see libvlc_video_filter_names_t
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_video_filter_add( libvlc_media_player_t *p_mediaplayer,
+                                             libvlc_video_filter_names_t filter,
+                                             libvlc_exception_t *p_e );
+
+/**
+ * Remove video filter
+ *
+ * \param p_mediaplayer the media player
+ * \param filter video filter @see libvlc_video_filter_names_t
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_video_filter_remove( libvlc_media_player_t *p_mediaplayer,
+                                                libvlc_video_filter_names_t filter,
+                                                libvlc_exception_t *p_e );
+
+/**
+ * Get info about video filter parameters
+ *
+ * \param p_instance libvlc instance
+ * \param filter video filter @see libvlc_video_filter_names_t
+ * \param p_e an initialized exception pointer
+ * \return linked list with parameters info
+ */
+VLC_PUBLIC_API libvlc_filter_parameter_list_t *libvlc_video_filter_get_parameters(
+        libvlc_instance_t *,
+        libvlc_video_filter_names_t,
+        libvlc_exception_t * );
+
+/**
+ * Set parameter for video filter
+ *
+ * \param p_mediaplayer the media player
+ * \param filter video filter @see libvlc_video_filter_names_t
+ * \param psz_parameter_name parameter name
+ * \param type type of value
+ * \param new_value new value of parameter
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_video_filter_set_parameter(
+        libvlc_media_player_t *,
+        libvlc_video_filter_names_t,
+        const char *,
+        libvlc_var_type_t,
+        libvlc_value_t,
+        libvlc_exception_t * );
+
+/**
+ * Set the brightness of video.
+ *
+ * \param p_mediaplayer the media player
+ * \param f_brightness new value of brightness for video, range: 0.0 - 2.0, default: 1.0
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_video_set_brightness( libvlc_media_player_t *p_mediaplayer,
+                                                 float f_brightness,
+                                                 libvlc_exception_t *p_e );
+
+/**
+ * Set the contrast of video.
+ *
+ * \param p_mediaplayer the media player
+ * \param f_contrast new value of constrast for video, range: 0.0 - 2.0, default: 1.0
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_video_set_contrast( libvlc_media_player_t *p_mediaplayer,
+                                               float f_contrast,
+                                               libvlc_exception_t *p_e );
+
+/**
+ * Set the gamma of video.
+ *
+ * \param p_mediaplayer the media player
+ * \param f_gamma new value of gamma for video, range: 0.0 - 10.0, default: 1.0
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_video_set_gamma( libvlc_media_player_t *p_mediaplayer,
+                                            float f_gamma,
+                                            libvlc_exception_t *p_e );
+
+/**
+ * Set the hue of video.
+ *
+ * \param p_mediaplayer the media player
+ * \param i_hue new value of hue for video, range: 0 - 360, default: 0
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_video_set_hue( libvlc_media_player_t *p_mediaplayer,
+                                          int i_hue,
+                                          libvlc_exception_t *p_e );
+
+/**
+ * Set the saturation of video.
+ *
+ * \param p_mediaplayer the media player
+ * \param f_saturation new value of saturation for video, range: 0.0 - 3.0, default: 1.0
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_video_set_saturation( libvlc_media_player_t *p_mediaplayer,
+                                                 float f_saturation,
+                                                 libvlc_exception_t *p_e );
+
+/** @} filter */
+
 /** @} video */
 
 /** \defgroup libvlc_audio libvlc_audio
@@ -885,6 +1175,82 @@ VLC_PUBLIC_API void libvlc_video_set_viewport( libvlc_instance_t *, const libvlc
  */
 
 /**
+ * Audio devices
+ *
+ */
+typedef enum libvlc_audio_device {
+    DEFAULT,
+    ALSA,
+    OSS,
+    SDL,
+    DIRECTX
+} libvlc_audio_device_t;
+
+/**
+ * Check if audio output is available.
+ *
+ * \param p_instance libvlc instance
+ * \param output_type type of audio output
+ */
+VLC_PUBLIC_API int libvlc_audio_device_available( libvlc_instance_t *,
+                                                  libvlc_audio_device_t );
+
+/**
+ * Set the audio output.
+ *
+ * \param p_instance libvlc instance
+ * \param output_type type of audio output
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_audio_device_set( libvlc_instance_t *,
+                                             libvlc_audio_device_t,
+                                             libvlc_exception_t * );
+
+/**
+ * Get count of ALSA devices.
+ *
+ * \param p_instance libvlc instance
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API int libvlc_audio_alsa_device_count( libvlc_instance_t *,
+                                                   libvlc_exception_t * );
+
+/**
+ * Get long name of alsa device, if not available short name given
+ *
+ * \param p_instance libvlc instance
+ * \param i_device alsa device
+ * \param p_e an initialized exception pointer
+ * \return name of device, need to be free after use
+ */
+VLC_PUBLIC_API char * libvlc_audio_alsa_device_longname( libvlc_instance_t *,
+                                                         int,
+                                                         libvlc_exception_t * );
+
+/**
+ * Get id name of alsa device
+ *
+ * \param p_instance libvlc instance
+ * \param i_device alsa device
+ * \param p_e an initialized exception pointer
+ * \return id name of device, use for setting device, need to be free after use
+ */
+VLC_PUBLIC_API char * libvlc_audio_alsa_device_id( libvlc_instance_t *,
+                                                   int,
+                                                   libvlc_exception_t * );
+
+/**
+ * Set ALSA device for using
+ *
+ * \param p_instance libvlc instance
+ * \param psz_device_id alsa device, get from libvlc_audio_alsa_device_id()
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_audio_alsa_device_set( libvlc_instance_t *,
+                                                  const char *,
+                                                  libvlc_exception_t * );
+
+/**
  * Toggle mute status.
  *
  * \param p_instance libvlc instance
@@ -938,6 +1304,16 @@ VLC_PUBLIC_API void libvlc_audio_set_volume( libvlc_instance_t *, int, libvlc_ex
 VLC_PUBLIC_API int libvlc_audio_get_track_count( libvlc_media_player_t *,  libvlc_exception_t * );
 
 /**
+ * Get the description of available audio tracks.
+ *
+ * \param p_mi media player
+ * \param p_e an initialized exception
+ * \return list with description of available audio tracks
+ */
+VLC_PUBLIC_API libvlc_track_description_t *
+        libvlc_audio_get_track_description( libvlc_media_player_t *,  libvlc_exception_t * );
+
+/**
  * Get current audio track.
  *
  * \param p_mi media player
@@ -973,6 +1349,126 @@ VLC_PUBLIC_API int libvlc_audio_get_channel( libvlc_instance_t *, libvlc_excepti
  */
 VLC_PUBLIC_API void libvlc_audio_set_channel( libvlc_instance_t *, int, libvlc_exception_t * );
 
+/**
+ * Get current audio device.
+ *
+ * \param p_instance vlc instance
+ * \param p_e an initialized exception pointer
+ * \return the audio device (int)
+ */
+VLC_PUBLIC_API int libvlc_audio_get_device( libvlc_instance_t *, libvlc_exception_t * );
+
+/**
+ * Set current audio device.
+ *
+ * \param p_instance vlc instance
+ * \param i_device the audio device (int)
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_audio_set_device( libvlc_instance_t *, int, libvlc_exception_t * );
+
+/**
+ * Audio filter names
+ */
+typedef enum libvlc_audio_filter_names_t {
+    EQUALIZER,
+    HEADPHONE_SPALIZATION,
+    PARAMETRIC_EQUALIZER,
+    SCALETEMPO,
+    SPATIALIZER,
+    VOLUME_NORMALIZER
+} libvlc_audio_filter_names_t;
+
+/**
+ * Audio preset names
+ *
+ * They can be used with libvlc_audio_equalizer_set_preset()
+ * You need to activate EQUALIZER filter
+ */
+typedef enum libvlc_audio_preset_names_t {
+    FLAT,
+    CLASSICAL,
+    CLUB,
+    DANCE,
+    FULLBASS,
+    FULLBASSTREBLE,
+    FULLTREBLE,
+    HEADPHONES,
+    LARGEHALL,
+    LIVE,
+    PARTY,
+    POP,
+    REGGAE,
+    ROCK,
+    SKA,
+    SOFT,
+    SOFTROCK,
+    TECHNO
+} libvlc_audio_preset_names_t;
+
+/**
+ * Add audio filter
+ *
+ * \param p_instance vlc instance
+ * \param filter audio filter
+ * \param p_e an initialized exception pointer
+ * \return true if succesfull (boolean)
+ */
+ VLC_PUBLIC_API int libvlc_audio_filter_add( libvlc_instance_t *,
+                                             libvlc_audio_filter_names_t,
+                                             libvlc_exception_t * );
+
+/**
+ * Remove audio filter
+ *
+ * \param p_instance vlc instance
+ * \param filter audio filter
+ * \param p_e an initialized exception pointer
+ * \return true if succesfull (boolean)
+ */
+VLC_PUBLIC_API int libvlc_audio_filter_remove( libvlc_instance_t *,
+                                               libvlc_audio_filter_names_t,
+                                               libvlc_exception_t * );
+
+/**
+ * Get info about audio filter parameters
+ *
+ * \param p_instance the media player
+ * \param filter audio filter @see libvlc_audio_filter_names_t
+ * \param p_e an initialized exception pointer
+ * \return linked list with parameters info
+ */
+VLC_PUBLIC_API libvlc_filter_parameter_list_t *libvlc_audio_filter_get_parameters(
+        libvlc_instance_t *,
+        libvlc_audio_filter_names_t,
+        libvlc_exception_t * );
+
+/**
+ * Set parameter for audio filter
+ *
+ * \param p_instance libvlc instance
+ * \param psz_parameter_name parameter name
+ * \param type type of value
+ * \param new_value new value of parameter
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_audio_filter_set_parameter(
+        libvlc_instance_t *,
+        const char *,
+        libvlc_var_type_t,
+        libvlc_value_t,
+        libvlc_exception_t * );
+
+/**
+ * Set audio preset. Equalizer has to be enable to take effect.
+ *
+ * \param p_instance vlc instance
+ * \param p_e an initialized exception pointer
+ */
+VLC_PUBLIC_API void libvlc_audio_equalizer_set_preset( libvlc_instance_t *,
+                                                       libvlc_audio_preset_names_t,
+                                                       libvlc_exception_t * );
+
 /** @} audio */
 
 /** @} media_player */
diff --git a/include/vlc/libvlc_events.h b/include/vlc/libvlc_events.h
index 1aa0f2f..524c349 100644
--- a/include/vlc/libvlc_events.h
+++ b/include/vlc/libvlc_events.h
@@ -66,6 +66,7 @@ typedef enum libvlc_event_type_t {
     libvlc_MediaPlayerPositionChanged,
     libvlc_MediaPlayerSeekableChanged,
     libvlc_MediaPlayerPausableChanged,
+    libvlc_MediaPlayerTitleChanged,
 
     libvlc_MediaListItemAdded,
     libvlc_MediaListWillAddItem,
@@ -142,6 +143,10 @@ typedef struct libvlc_event_t
         {
             libvlc_time_t new_pausable;
         } media_player_pausable_changed;
+        struct
+        {
+            int new_title;
+        } media_player_title_changed;
 
         /* media list */
         struct
diff --git a/include/vlc/libvlc_structures.h b/include/vlc/libvlc_structures.h
index 76427ad..7cfc181 100644
--- a/include/vlc/libvlc_structures.h
+++ b/include/vlc/libvlc_structures.h
@@ -31,6 +31,10 @@
 
 #include <stdint.h>
 
+#ifndef __cplusplus
+# include <stdbool.h>
+#endif
+
 # ifdef __cplusplus
 extern "C" {
 # endif
@@ -38,6 +42,17 @@ extern "C" {
 /** This structure is opaque. It represents a libvlc instance */
 typedef struct libvlc_instance_t libvlc_instance_t;
 
+/**
+ * LIBVLC value structure
+ */
+typedef union
+{
+    int             i_int;
+    bool            b_bool;
+    float           f_float;
+    char *          psz_string;
+} libvlc_value_t;
+
 /*****************************************************************************
  * Exceptions
  *****************************************************************************/
@@ -209,6 +224,55 @@ typedef struct libvlc_playlist_item_t
 
 /**@} */
 
+/*****************************************************************************
+ * Media Player
+ *****************************************************************************/
+/** \defgroup libvlc_media_player libvlc_media_player
+ * \ingroup libvlc
+ * LibVLC Media Player, object that let you play a media
+ * in a libvlc_drawable_t
+ * @{
+ */
+
+/**
+ * Description for video, audio tracks and subtitles. It contains
+ * id, name (description string) and pointer to next record.
+ */
+ typedef struct libvlc_track_description_t
+ {
+     int   i_id;
+     char *psz_name;
+     struct libvlc_track_description_t *p_next;
+
+ } libvlc_track_description_t;
+
+/**
+ * Type of variable - audio/video filters
+ */
+typedef enum libvlc_var_type_t
+{
+    LIBVLC_BOOL,
+    LIBVLC_INT,
+    LIBVLC_FLOAT,
+    LIBVLC_STRING
+} libvlc_var_type_t;
+
+/**
+ * Description for audio/video filter
+ * It contains info about filter parameters
+ */
+ typedef struct libvlc_filter_parameter_list_t
+ {
+     char *psz_parameter_name;
+     char *psz_description;
+     libvlc_var_type_t var_type;
+     libvlc_value_t default_value;
+     libvlc_value_t min_value;
+     libvlc_value_t max_value;
+     struct libvlc_filter_parameter_list_t *p_next;
+ } libvlc_filter_parameter_list_t;
+
+/**@} */
 
 /*****************************************************************************
  * Video
diff --git a/modules/gui/qt4/components/interface_widgets.cpp b/modules/gui/qt4/components/interface_widgets.cpp
index 56987d1..9424a0b 100644
--- a/modules/gui/qt4/components/interface_widgets.cpp
+++ b/modules/gui/qt4/components/interface_widgets.cpp
@@ -1114,6 +1114,7 @@ void FullscreenControllerWidget::planHideFSC()
 #endif
 }
 
+
 /**
  * Hidding fullscreen controller slowly
  * Linux: need composite manager
diff --git a/modules/video_filter/colorthres.c b/modules/video_filter/colorthres.c
index b27b7a5..a8cae53 100644
--- a/modules/video_filter/colorthres.c
+++ b/modules/video_filter/colorthres.c
@@ -78,6 +78,8 @@ vlc_module_begin();
     add_integer( CFG_PREFIX "similaritythres", 15, NULL,
                  _("Similarity threshold"), "", false );
     set_callbacks( Create, Destroy );
+
+    add_shortcut( "colorthres" );
 vlc_module_end();
 
 static const char *const ppsz_filter_options[] = {
diff --git a/src/control/audio.c b/src/control/audio.c
index 22a886e..26548bc 100644
--- a/src/control/audio.c
+++ b/src/control/audio.c
@@ -23,11 +23,25 @@
  *****************************************************************************/
 
 #include "libvlc_internal.h"
+#include "../../modules/audio_filter/equalizer_presets.h"
 #include <vlc/libvlc.h>
 
 #include <vlc_input.h>
 #include <vlc_aout.h>
 
+/* Names of filters
+ * it has to be written according to enum libvlc_audio_filter_names_t
+ * in libvlc.h
+ */
+static const char * ppsz_filter_names[] = {
+    "equalizer",                    /* EQUALIZER */
+    "headphone_channel_mixer",      /* HEADPHONE_SPALIZATION */
+    "param_eq",                     /* PARAMETRIC_EQUALIZER */
+    "scaletempo",                   /* SCALETEMPO */
+    "spatializer",                  /* SPATIALIZER */
+    "normvol",                      /* VOLUME_NORMALIZER */
+};
+
 
 /*
  * Remember to release the returned aout_instance_t since it is locked at
@@ -36,6 +50,9 @@
 static aout_instance_t *GetAOut( libvlc_instance_t *p_instance,
                                  libvlc_exception_t *p_exception )
 {
+    if( !p_instance )
+        return NULL;
+
     aout_instance_t * p_aout = NULL;
 
     p_aout = vlc_object_find( p_instance->p_libvlc_int, VLC_OBJECT_AOUT, FIND_CHILD );
@@ -48,6 +65,171 @@ static aout_instance_t *GetAOut( libvlc_instance_t *p_instance,
     return p_aout;
 }
 
+/********************************************
+ * Check if audio output is available.
+ * Return true/1 if available
+ ********************************************/
+int libvlc_audio_device_available( libvlc_instance_t *p_instance,
+                                   libvlc_audio_device_t output_type )
+{
+#ifdef WIN32
+    // directx
+#else
+    switch( output_type )
+    {
+        case DEFAULT:
+            return 1;
+        case ALSA:
+            return module_Exists( p_instance->p_libvlc_int, "alsa");
+        case OSS:
+            return module_Exists( p_instance->p_libvlc_int, "oss");
+        case SDL:
+            return module_Exists( p_instance->p_libvlc_int, "aout_sdl");
+        default:
+            return 0;
+    }
+#endif
+    return 0;
+}
+
+/******************************
+ * Set the audio output.
+ *******************************/
+void libvlc_audio_device_set( libvlc_instance_t *p_instance,
+                              libvlc_audio_device_t output_type,
+                              libvlc_exception_t *p_e )
+{
+#ifdef WIN32
+    // directx
+#else
+    switch( output_type )
+    {
+        case DEFAULT:
+            config_PutPsz( p_instance->p_libvlc_int, "aout", "" );
+            break;
+        case ALSA:
+            config_PutPsz( p_instance->p_libvlc_int, "aout", "alsa" );
+            break;
+        case OSS:
+            config_PutPsz( p_instance->p_libvlc_int, "aout", "oss" );
+            break;
+        case SDL:
+            config_PutPsz( p_instance->p_libvlc_int, "aout", "aout_sdl" );
+            break;
+        default:
+            libvlc_exception_raise( p_e, "Unknown audio output" );
+            break;
+    }
+#endif
+}
+
+/****************************
+ * Get count of ALSA devices.
+ *****************************/
+int libvlc_audio_alsa_device_count( libvlc_instance_t *p_instance,
+                                    libvlc_exception_t *p_e )
+{
+    module_config_t *p_module_config = config_FindConfig(
+        ( vlc_object_t * ) p_instance->p_libvlc_int, "alsadev" );
+    if( p_module_config && p_module_config->pf_update_list )
+    {
+        vlc_value_t val;
+        val.psz_string = strdup( p_module_config->value.psz );
+
+        p_module_config->pf_update_list(
+            ( vlc_object_t * ) p_instance->p_libvlc_int, "alsadev", val, val, NULL );
+        free( val.psz_string );
+
+        return p_module_config->i_list;
+    }
+    libvlc_exception_raise( p_e, "Alsa module not available" );
+    return 0;
+}
+
+/********************************
+ * Get long name of alsa device
+ *********************************/
+char * libvlc_audio_alsa_device_longname( libvlc_instance_t *p_instance,
+                                          int i_device,
+                                          libvlc_exception_t *p_e )
+{
+    module_config_t *p_module_config = config_FindConfig(
+        ( vlc_object_t * ) p_instance->p_libvlc_int, "alsadev" );
+    if( p_module_config )
+    {
+        // refresh if there arent devices
+        if( p_module_config->i_list < 2 && p_module_config->pf_update_list )
+        {
+            vlc_value_t val;
+            val.psz_string = strdup( p_module_config->value.psz );
+
+            p_module_config->pf_update_list(
+                ( vlc_object_t * ) p_instance->p_libvlc_int, "alsadev", val, val, NULL );
+            free( val.psz_string );
+        }
+
+        if( i_device >= 0 && i_device < p_module_config->i_list )
+        {
+          if( p_module_config->ppsz_list_text[i_device] )
+              return strdup( p_module_config->ppsz_list_text[i_device] );
+          else
+              return strdup( p_module_config->ppsz_list[i_device] );
+        }
+    }
+    libvlc_exception_raise( p_e, "Alsa module not available" );
+    return NULL;
+}
+
+/********************************
+ * Get id name of alsa device
+ *********************************/
+char * libvlc_audio_alsa_device_id( libvlc_instance_t *p_instance,
+                                    int i_device,
+                                    libvlc_exception_t *p_e )
+{
+    module_config_t *p_module_config = config_FindConfig(
+        ( vlc_object_t * ) p_instance->p_libvlc_int, "alsadev" );
+    if( p_module_config )
+    {
+        // refresh if there arent devices
+        if( p_module_config->i_list < 2 && p_module_config->pf_update_list )
+        {
+            vlc_value_t val;
+            val.psz_string = strdup( p_module_config->value.psz );
+
+            p_module_config->pf_update_list(
+                ( vlc_object_t * ) p_instance->p_libvlc_int, "alsadev", val, val, NULL );
+            free( val.psz_string );
+        }
+
+        if( i_device >= 0 && i_device < p_module_config->i_list )
+        {
+              return strdup( p_module_config->ppsz_list[i_device] );
+        }
+    }
+    libvlc_exception_raise( p_e, "Alsa module not available" );
+    return NULL;
+}
+
+/*****************************
+ * Set ALSA device for using
+ *****************************/
+VLC_PUBLIC_API void libvlc_audio_alsa_device_set( libvlc_instance_t *p_instance,
+                                                  const char *psz_device_id,
+                                                  libvlc_exception_t *p_e )
+{
+    char *psz_aout = config_GetPsz( p_instance->p_libvlc_int, "aout" );
+    if( strcmp( psz_aout, "alsa" ) != 0 )
+    {
+        //set alsa output
+        libvlc_audio_device_set( p_instance, ALSA, p_e );
+    }
+    free( psz_aout );
+
+    if( psz_device_id )
+        config_PutPsz( p_instance->p_libvlc_int, "alsadev", psz_device_id );
+}
+
 
 /*****************************************************************************
  * libvlc_audio_get_mute : Get the volume state, true if muted
@@ -135,6 +317,16 @@ int libvlc_audio_get_track_count( libvlc_media_player_t *p_mi,
 }
 
 /*****************************************************************************
+ * libvlc_audio_get_track_description : Get the description of available audio tracks
+ *****************************************************************************/
+libvlc_track_description_t *
+        libvlc_audio_get_track_description( libvlc_media_player_t *p_mi,
+                                            libvlc_exception_t *p_e )
+{
+    return libvlc_get_track_description( p_mi, "audio-es", p_e);
+}
+
+/*****************************************************************************
  * libvlc_audio_get_track : Get the current audio track
  *****************************************************************************/
 int libvlc_audio_get_track( libvlc_media_player_t *p_mi,
@@ -260,3 +452,167 @@ void libvlc_audio_set_channel( libvlc_instance_t *p_instance, int i_channel,
         vlc_object_release( p_aout );
     }
 }
+
+/*****************************************************************************
+ * libvlc_audio_get_device : Get the current audio device
+ *****************************************************************************/
+int libvlc_audio_get_device( libvlc_instance_t *p_instance,
+                             libvlc_exception_t *p_e )
+{
+    aout_instance_t *p_aout = GetAOut( p_instance, p_e );
+    if( p_aout )
+    {
+        vlc_value_t val;
+
+        var_Get( p_aout, "audio-device", &val );
+        vlc_object_release( p_aout );
+        return val.i_int;
+    }
+    return -1;
+}
+
+/*****************************************************************************
+ * libvlc_audio_set_device : Set the audio device
+ *****************************************************************************/
+void libvlc_audio_set_device( libvlc_instance_t *p_instance, int i_device,
+                              libvlc_exception_t *p_e )
+{
+    aout_instance_t *p_aout = GetAOut( p_instance, p_e );
+    if( p_aout )
+    {
+        vlc_value_t val;
+        int i_ret = -1;
+
+        val.i_int = i_device;
+        switch( i_device )
+        {
+            case AOUT_VAR_MONO:
+            case AOUT_VAR_STEREO:
+            case AOUT_VAR_2F2R:
+            case AOUT_VAR_3F2R:
+            case AOUT_VAR_5_1:
+            case AOUT_VAR_6_1:
+            case AOUT_VAR_7_1:
+            case AOUT_VAR_SPDIF:
+                i_ret = var_Set( p_aout, "audio-device", val );
+                if( i_ret < 0 )
+                {
+                    libvlc_exception_raise( p_e, "Failed setting audio device" );
+                    vlc_object_release( p_aout );
+                    return;
+                }
+                vlc_object_release( p_aout );
+                return; /* Found */
+            default:
+                libvlc_exception_raise( p_e, "Audio device out of range" );
+                break;
+        }
+        vlc_object_release( p_aout );
+    }
+}
+
+/**
+ * Add audio filter
+ */
+int libvlc_audio_filter_add( libvlc_instance_t *p_instance,
+                             libvlc_audio_filter_names_t filter,
+                             libvlc_exception_t *p_e )
+{
+    aout_instance_t *p_aout = GetAOut( p_instance, p_e );
+    if( p_aout )
+    {
+        aout_EnableFilter( ( vlc_object_t * ) p_aout->p_libvlc, ppsz_filter_names[filter], true );
+        vlc_object_release( p_aout );
+        return 1;
+    }
+    return 0;
+}
+
+/**
+ * Remove audio filter
+ */
+int libvlc_audio_filter_remove( libvlc_instance_t *p_instance,
+                                libvlc_audio_filter_names_t filter,
+                                libvlc_exception_t *p_e )
+{
+    aout_instance_t *p_aout = GetAOut( p_instance, p_e );
+    if( p_aout )
+    {
+        aout_EnableFilter( ( vlc_object_t * ) p_aout->p_libvlc, ppsz_filter_names[filter], false );
+        vlc_object_release( p_aout );
+        return 1;
+    }
+    return 0;
+}
+
+/**
+ * Get info about audio filter parameters
+ */
+libvlc_filter_parameter_list_t *libvlc_audio_filter_get_parameters(
+        libvlc_instance_t *p_instance,
+        libvlc_audio_filter_names_t filter,
+        libvlc_exception_t *p_e )
+{
+    module_t * p_module = module_Find( p_instance->p_libvlc_int, ppsz_filter_names[filter] );
+    if( !p_module )
+    {
+        libvlc_exception_raise( p_e, "Unknown module" );
+        return NULL;
+    }
+    libvlc_filter_parameter_list_t *p_list = libvlc_filter_get_parameters( p_module, p_e );
+    module_Put( p_module );
+
+    return p_list;
+}
+
+/**
+ * Set parameter for audio filter
+ */
+void libvlc_audio_filter_set_parameter(
+        libvlc_instance_t *p_instance,
+        const char *psz_parameter_name,
+        libvlc_var_type_t type,
+        libvlc_value_t new_value,
+        libvlc_exception_t *p_e )
+{
+    libvlc_filter_set_parameter(
+        p_instance,
+        psz_parameter_name,
+        type,
+        new_value );
+}
+
+/**
+ * Set audio preset. Equalizer has to be enable to take effect.
+ */
+void libvlc_audio_equalizer_set_preset( libvlc_instance_t * p_instance,
+                                        libvlc_audio_preset_names_t preset,
+                                        libvlc_exception_t *p_e )
+{
+    aout_instance_t *p_aout = GetAOut( p_instance, p_e );
+    if( p_aout )
+    {
+        /* MAX number of chars in one number should be 8
+           we can use 10 for sure*/
+        char *psz_values = malloc( EQZ_BANDS_MAX * 10 * sizeof( char ) );
+        if( !psz_values )
+        {
+            vlc_object_release( p_aout );
+            libvlc_exception_raise( p_e, "Not enough memory" );
+            return;
+        }
+
+        char *psz_writing = psz_values;
+        for( int i = 0 ; i < EQZ_BANDS_MAX ; i++ )
+        {
+            sprintf( psz_writing, "%1.3f   ", eqz_preset_10b[preset]->f_amp[i] );
+            psz_writing += 8; /* move 8 spaces forward for next write */
+        }
+
+        var_SetString( p_aout, "equalizer-bands", psz_values );
+        var_SetFloat( p_aout, "equalizer-preamp",
+                      eqz_preset_10b[preset]->f_preamp );
+        vlc_object_release( p_aout );
+        free( psz_values );
+    }
+}
diff --git a/src/control/event.c b/src/control/event.c
index a98f23c..4864a09 100644
--- a/src/control/event.c
+++ b/src/control/event.c
@@ -249,6 +249,7 @@ static const char event_type_to_name[][35] =
     EVENT(libvlc_MediaPlayerBackward),
     EVENT(libvlc_MediaPlayerEndReached),
     EVENT(libvlc_MediaPlayerTimeChanged),
+    EVENT(libvlc_MediaPlayerTitleChanged),
     EVENT(libvlc_MediaPlayerPositionChanged),
     EVENT(libvlc_MediaPlayerSeekableChanged),
     EVENT(libvlc_MediaPlayerPausableChanged),
diff --git a/src/control/libvlc_internal.h b/src/control/libvlc_internal.h
index a291858..02e1843 100644
--- a/src/control/libvlc_internal.h
+++ b/src/control/libvlc_internal.h
@@ -144,15 +144,31 @@ struct libvlc_media_list_view_t
     void (*pf_ml_item_removed)(const libvlc_event_t *, libvlc_media_list_view_t *);
 };
 
+typedef struct libvlc_filter_option_t
+{
+    char *                             psz_option;
+    libvlc_value_t                        value;
+    libvlc_var_type_t                  type;
+    struct libvlc_filter_option_t *    p_next_option;
+} libvlc_filter_option_t;
+
+typedef struct libvlc_filter_chain_t
+{
+    const char *                     psz_filter;
+    libvlc_filter_option_t *         p_option;
+    struct libvlc_filter_chain_t *   p_next_filter;
+} libvlc_filter_chain_t;
+
 struct libvlc_media_player_t
 {
-    int                i_refcount;
-    vlc_mutex_t        object_lock;
-    input_thread_t *   p_input_thread;
+    int                         i_refcount;
+    vlc_mutex_t                 object_lock;
+    input_thread_t *            p_input_thread;
     struct libvlc_instance_t *  p_libvlc_instance; /* Parent instance */
-    libvlc_media_t * p_md; /* current media descriptor */
+    libvlc_media_t *            p_md;              /* current media descriptor */
     libvlc_event_manager_t *    p_event_manager;
     libvlc_drawable_t           drawable;
+    libvlc_filter_chain_t *     p_filter_chain;
     
     bool        b_own_its_input_thread;
 };
@@ -339,6 +355,29 @@ void libvlc_event_send(
         libvlc_event_manager_t * p_em,
         libvlc_event_t * p_event );
 
+/* Media player - audio, video */
+libvlc_track_description_t * libvlc_get_track_description(
+        libvlc_media_player_t *p_mi,
+        const char *psz_variable,
+        libvlc_exception_t *p_e );
+
+/**
+ * Get info about parameters for module
+ */
+libvlc_filter_parameter_list_t *libvlc_filter_get_parameters(
+        module_t *p_module,
+        libvlc_exception_t *p_e );
+
+/**
+ * Set audio/video filter parameter
+ * This function set parameter in config
+ * and dont apply it directly(immediately)
+ */
+void libvlc_filter_set_parameter(
+        libvlc_instance_t *,
+        const char *psz_parameter_name,
+        libvlc_var_type_t type,
+        libvlc_value_t new_value );
 
 /* Exception shorcuts */
 
diff --git a/src/control/media_player.c b/src/control/media_player.c
index a90d177..081a441 100644
--- a/src/control/media_player.c
+++ b/src/control/media_player.c
@@ -29,6 +29,8 @@
 #include <vlc_vout.h>
 #include "libvlc.h"
 
+#include <limits.h>
+
 static void
 input_state_changed( const vlc_event_t * event, void * p_userdata );
 
@@ -324,6 +326,7 @@ libvlc_media_player_new( libvlc_instance_t * p_libvlc_instance,
     p_mi->drawable = 0;
     p_mi->p_libvlc_instance = p_libvlc_instance;
     p_mi->p_input_thread = NULL;
+    p_mi->p_filter_chain = NULL;
     /* refcount strategy:
      * - All items created by _new start with a refcount set to 1
      * - Accessor _release decrease the refcount by 1, if after that
@@ -370,6 +373,8 @@ libvlc_media_player_new( libvlc_instance_t * p_libvlc_instance,
     libvlc_event_manager_register_event_type( p_mi->p_event_manager,
             libvlc_MediaPlayerTimeChanged, p_e );
     libvlc_event_manager_register_event_type( p_mi->p_event_manager,
+            libvlc_MediaPlayerTitleChanged, p_e );
+    libvlc_event_manager_register_event_type( p_mi->p_event_manager,
             libvlc_MediaPlayerSeekableChanged, p_e );
     libvlc_event_manager_register_event_type( p_mi->p_event_manager,
             libvlc_MediaPlayerPausableChanged, p_e );
@@ -467,6 +472,23 @@ void libvlc_media_player_destroy( libvlc_media_player_t *p_mi )
 
     libvlc_media_release( p_mi->p_md );
 
+    libvlc_filter_chain_t * p_filter = p_mi->p_filter_chain;
+    while( p_filter )
+    {
+        libvlc_filter_option_t *p_option = p_filter->p_option;
+        while ( p_option )
+        {
+            free( p_option->psz_option );
+            libvlc_filter_option_t *p_o = p_option;
+            p_option = p_option->p_next_option;
+            free( p_o );
+        }
+        libvlc_filter_chain_t *p_f = p_filter;
+        p_filter = p_filter->p_next_filter;
+        free( p_f->psz_filter );
+        free( p_f );
+    }
+
     free( p_mi );
 }
 
@@ -856,7 +878,7 @@ int libvlc_media_player_get_chapter(
 
     p_input_thread = libvlc_get_input_thread ( p_mi, p_e );
     if( !p_input_thread )
-        return -1.0;
+        return -1;
 
     var_Get( p_input_thread, "chapter", &val );
     vlc_object_release( p_input_thread );
@@ -873,7 +895,7 @@ int libvlc_media_player_get_chapter_count(
 
     p_input_thread = libvlc_get_input_thread ( p_mi, p_e );
     if( !p_input_thread )
-        return -1.0;
+        return -1;
 
     var_Change( p_input_thread, "chapter", VLC_VAR_CHOICESCOUNT, &val, NULL );
     vlc_object_release( p_input_thread );
@@ -881,6 +903,122 @@ int libvlc_media_player_get_chapter_count(
     return val.i_int;
 }
 
+int libvlc_media_player_get_chapter_count_for_title(
+                                 libvlc_media_player_t *p_mi,
+                                 int i_title,
+                                 libvlc_exception_t *p_e )
+{
+    input_thread_t *p_input_thread;
+    vlc_value_t val;
+
+    p_input_thread = libvlc_get_input_thread ( p_mi, p_e );
+    if( !p_input_thread )
+        return -1;
+
+    char *psz_name = malloc( sizeof("title ") + 5 );
+    sprintf( psz_name,  "title %2i", i_title );
+    var_Change( p_input_thread, psz_name, VLC_VAR_CHOICESCOUNT, &val, NULL );
+    vlc_object_release( p_input_thread );
+    free( psz_name );
+
+    return val.i_int;
+}
+
+void libvlc_media_player_set_title(
+                                 libvlc_media_player_t *p_mi,
+                                 int i_title,
+                                 libvlc_exception_t *p_e )
+{
+    input_thread_t *p_input_thread;
+    vlc_value_t val;
+    val.i_int = i_title;
+
+    p_input_thread = libvlc_get_input_thread ( p_mi, p_e);
+    if( !p_input_thread )
+        return;
+
+    var_Set( p_input_thread, "title", val );
+    vlc_object_release( p_input_thread );
+
+    //send event
+    libvlc_event_t event;
+    event.type = libvlc_MediaPlayerTitleChanged;
+    event.u.media_player_title_changed.new_title = i_title;
+    libvlc_event_send( p_mi->p_event_manager, &event );
+}
+
+int libvlc_media_player_get_title(
+                                 libvlc_media_player_t *p_mi,
+                                 libvlc_exception_t *p_e )
+{
+    input_thread_t *p_input_thread;
+    vlc_value_t val;
+
+    p_input_thread = libvlc_get_input_thread ( p_mi, p_e );
+    if( !p_input_thread )
+        return -1;
+
+    var_Get( p_input_thread, "title", &val );
+    vlc_object_release( p_input_thread );
+
+    return val.i_int;
+}
+
+int libvlc_media_player_get_title_count(
+                                 libvlc_media_player_t *p_mi,
+                                 libvlc_exception_t *p_e )
+{
+    input_thread_t *p_input_thread;
+    vlc_value_t val;
+
+    p_input_thread = libvlc_get_input_thread ( p_mi, p_e );
+    if( !p_input_thread )
+        return -1;
+
+    var_Change( p_input_thread, "title", VLC_VAR_CHOICESCOUNT, &val, NULL );
+    vlc_object_release( p_input_thread );
+
+    return val.i_int;
+}
+
+void libvlc_media_player_next_chapter(
+                                 libvlc_media_player_t *p_mi,
+                                 libvlc_exception_t *p_e )
+{
+    input_thread_t *p_input_thread;
+
+    p_input_thread = libvlc_get_input_thread ( p_mi, p_e);
+    if( !p_input_thread )
+        return;
+
+    int i_type = var_Type( p_input_thread, "next-chapter" );
+    vlc_value_t val;
+    val.b_bool = true;
+    var_Set( p_input_thread, (i_type & VLC_VAR_TYPE) != 0 ?
+                            "next-chapter":"next-title", val );
+
+    vlc_object_release( p_input_thread );
+}
+
+void libvlc_media_player_previous_chapter(
+                                 libvlc_media_player_t *p_mi,
+                                 libvlc_exception_t *p_e )
+{
+    input_thread_t *p_input_thread;
+
+    p_input_thread = libvlc_get_input_thread ( p_mi, p_e);
+    if( !p_input_thread )
+        return;
+
+   int i_type = var_Type( p_input_thread, "next-chapter" );
+   vlc_value_t val;
+   val.b_bool = true;
+   var_Set( p_input_thread, (i_type & VLC_VAR_TYPE) != 0 ?
+                            "prev-chapter":"prev-title", val );
+
+    vlc_object_release( p_input_thread );
+}
+
 float libvlc_media_player_get_fps(
                                  libvlc_media_player_t *p_mi,
                                  libvlc_exception_t *p_e)
@@ -1013,3 +1151,286 @@ int libvlc_media_player_can_pause( libvlc_media_player_t *p_mi,
 
     return val.b_bool;
 }
+
+/* internal function, used by audio, video */
+libvlc_track_description_t *
+        libvlc_get_track_description( libvlc_media_player_t *p_mi,
+                                      const char *psz_variable,
+                                      libvlc_exception_t *p_e )
+{
+    input_thread_t *p_input = libvlc_get_input_thread( p_mi, p_e );
+
+    if( !p_input )
+        return NULL;
+
+    vlc_value_t val_list, text_list;
+    var_Change( p_input, psz_variable, VLC_VAR_GETLIST, &val_list, &text_list);
+
+    if( val_list.p_list->i_count <= 0 ) /* no tracks */
+        return NULL;
+
+    libvlc_track_description_t *p_track_description, *p_actual, *p_previous;
+    p_track_description = ( libvlc_track_description_t * )
+        malloc( sizeof( libvlc_track_description_t ) );
+    if ( !p_track_description )
+    {
+        var_Change( p_input, psz_variable, VLC_VAR_FREELIST, &val_list, &text_list);
+        vlc_object_release( p_input );
+        libvlc_exception_raise( p_e, "no enough memory" );
+        return NULL;
+    }
+    p_actual = p_track_description;
+    p_previous = NULL;
+    for( int i = 0; i < val_list.p_list->i_count; i++ )
+    {
+        if( !p_actual )
+        {
+            p_actual = ( libvlc_track_description_t * )
+                malloc( sizeof( libvlc_track_description_t ) );
+            if ( !p_actual )
+            {
+                libvlc_track_description_release( p_track_description );
+                var_Change( p_input, psz_variable, VLC_VAR_FREELIST, &val_list, &text_list);
+                vlc_object_release( p_input );
+                libvlc_exception_raise( p_e, "no enough memory" );
+                return NULL;
+            }
+        }
+        p_actual->i_id = val_list.p_list->p_values[i].i_int;
+        p_actual->psz_name = strdup( text_list.p_list->p_values[i].psz_string );
+        p_actual->p_next = NULL;
+        if( p_previous )
+            p_previous->p_next = p_actual;
+        p_previous = p_actual;
+        p_actual =  NULL;
+    }
+    var_Change( p_input, psz_variable, VLC_VAR_FREELIST, &val_list, &text_list);
+    vlc_object_release( p_input );
+
+    return p_track_description;
+}
+
+void libvlc_track_description_release( libvlc_track_description_t *p_track_description )
+{
+    libvlc_track_description_t *p_actual, *p_before;
+    p_actual = p_track_description;
+
+    while ( p_actual )
+    {
+        free( p_actual->psz_name );
+        p_before = p_actual;
+        p_actual = p_before->p_next;
+        free( p_before );
+    }
+}
+
+/**
+ * Get info about parameters for module
+ */
+libvlc_filter_parameter_list_t *libvlc_filter_get_parameters(
+              module_t *p_module,
+              libvlc_exception_t *p_e )
+{
+    unsigned confsize;
+    module_config_t *p_config = module_GetConfig( p_module, &confsize );
+
+    libvlc_filter_parameter_list_t *p_list = NULL,
+                                   *p_actual = NULL;
+    if( p_config )
+    {
+        for( size_t i = 0; i < confsize; i++ )
+        {
+            module_config_t *p_item = p_config + i;
+            int i_type = p_item->i_type & CONFIG_ITEM;
+            // check if we have defined psz_type, then it is variable
+            if( p_item->psz_name )
+            {
+                // get name format: module-parameter_name
+                const char *psz_name = p_item->psz_name;
+                const char *psz_description;
+                if( p_item->psz_longtext )
+                    psz_description = p_item->psz_longtext;
+                else if ( p_item->psz_text )
+                    psz_description = p_item->psz_text;
+                else
+                    psz_description = "";
+                libvlc_filter_parameter_list_t *p_new = NULL;
+                switch( i_type )
+                {
+                    case CONFIG_ITEM_BOOL:
+                    {
+                        p_new = ( libvlc_filter_parameter_list_t * )
+                            malloc( sizeof ( libvlc_filter_parameter_list_t ) );
+                        if( !p_new )
+                        {
+                            libvlc_filter_parameters_release( p_list );
+                            libvlc_exception_raise( p_e, "Not enough memory" );
+                            return NULL;
+                        }
+                        p_new->psz_parameter_name = strdup( psz_name );
+                        p_new->psz_description = strdup( psz_description );
+                        p_new->var_type = LIBVLC_BOOL;
+                        libvlc_value_t def_val;
+                        def_val.b_bool = p_item->orig.i;  /* bool from int */
+                        p_new->default_value = def_val;
+                        p_new->p_next = NULL;
+                        break;
+                    }
+                    case CONFIG_ITEM_INTEGER:
+                    {
+                        p_new = ( libvlc_filter_parameter_list_t * )
+                            malloc( sizeof ( libvlc_filter_parameter_list_t ) );
+                        if( !p_new )
+                        {
+                            libvlc_filter_parameters_release( p_list );
+                            libvlc_exception_raise( p_e, "Not enough memory" );
+                            return NULL;
+                        }
+                        p_new->psz_parameter_name = strdup( psz_name );
+                        p_new->psz_description = strdup( psz_description );
+                        p_new->var_type = LIBVLC_INT;
+                        libvlc_value_t def_val, min_val, max_val;
+                        def_val.i_int = p_item->orig.i;
+                        if( p_item->min.i != p_item->max.i )
+                        {
+                            min_val.i_int = p_item->min.i;
+                            max_val.i_int = p_item->max.i;
+                        }
+                        else
+                        {
+                            min_val.i_int = INT_MIN;
+                            max_val.i_int = INT_MAX;
+                        }
+                        p_new->default_value = def_val;
+                        p_new->min_value = min_val;
+                        p_new->max_value = max_val;
+                        p_new->p_next = NULL;
+                        break;
+                    }
+                    case CONFIG_ITEM_FLOAT:
+                    {
+                        p_new = ( libvlc_filter_parameter_list_t * )
+                            malloc( sizeof ( libvlc_filter_parameter_list_t ) );
+                        if( !p_new )
+                        {
+                            libvlc_filter_parameters_release( p_list );
+                            libvlc_exception_raise( p_e, "Not enough memory" );
+                            return NULL;
+                        }
+                        p_new->psz_parameter_name = strdup( psz_name );
+                        p_new->psz_description = strdup( psz_description );
+                        p_new->var_type = LIBVLC_FLOAT;
+                        libvlc_value_t def_val, min_val, max_val;
+                        def_val.f_float = p_item->orig.f;
+                        if( p_item->min.f != p_item->max.f )
+                        {
+                            min_val.f_float = p_item->min.f;
+                            max_val.f_float = p_item->max.f;
+                        }
+                        else
+                        {
+                            min_val.f_float = ( float ) INT_MIN; /* not nice, but should be ok */
+                            max_val.f_float = ( float ) INT_MAX;
+                        }
+                        p_new->default_value = def_val;
+                        p_new->min_value = min_val;
+                        p_new->max_value = max_val;
+                        p_new->p_next = NULL;
+                        break;
+                    }
+                    case CONFIG_ITEM_STRING:
+                    {
+                        p_new = ( libvlc_filter_parameter_list_t * )
+                            malloc( sizeof ( libvlc_filter_parameter_list_t ) );
+                        if( !p_new )
+                        {
+                            libvlc_filter_parameters_release( p_list );
+                            libvlc_exception_raise( p_e, "Not enough memory" );
+                            return NULL;
+                        }
+                        p_new->psz_parameter_name = strdup( psz_name );
+                        p_new->psz_description = strdup( psz_description );
+                        p_new->var_type = LIBVLC_STRING;
+                        libvlc_value_t def_val, min_val, max_val;
+                        if( p_item->orig.psz )
+                            def_val.psz_string = strdup( p_item->orig.psz );
+                        else
+                            def_val.psz_string = NULL;
+                        min_val.psz_string = NULL;
+                        max_val.psz_string = NULL;
+                        p_new->default_value = def_val;
+                        p_new->min_value = min_val;
+                        p_new->max_value = max_val;
+                        p_new->p_next = NULL;
+                        break;
+                    }
+                    default:
+                        break;
+                }
+                if( p_new )
+                {
+                    if( !p_actual)  /* first item */
+                    {
+                        p_actual = p_new;
+                        p_list = p_actual;
+                    }
+                    else
+                    {
+                        p_actual->p_next = p_new;
+                        p_actual = p_actual->p_next;
+                    }
+                }
+            }
+        }
+        module_PutConfig( p_config );
+    }
+    return p_list;
+}
+
+/**
+ * Set audio/video filter parameter
+ */
+void libvlc_filter_set_parameter(
+        libvlc_instance_t *p_libvlc_instance,
+        const char *psz_parameter_name,
+        libvlc_var_type_t type,
+        libvlc_value_t new_value )
+{
+    switch( type )
+    {
+        case LIBVLC_BOOL:
+        case LIBVLC_INT:
+            config_PutInt( p_libvlc_instance->p_libvlc_int, psz_parameter_name, new_value.i_int );
+            break;
+        case LIBVLC_FLOAT:
+            config_PutFloat( p_libvlc_instance->p_libvlc_int, psz_parameter_name, new_value.f_float );
+            break;
+        case LIBVLC_STRING:
+            config_PutPsz( p_libvlc_instance->p_libvlc_int, psz_parameter_name, new_value.psz_string );
+            break;
+        default:
+            break;
+    }
+}
+
+/**
+ * Release (free) libvlc_filter_parameter_list_t
+ */
+void libvlc_filter_parameters_release( libvlc_filter_parameter_list_t *p_parameter_list )
+{
+    libvlc_filter_parameter_list_t *p_actual;
+    while( p_parameter_list )
+    {
+        p_actual = p_parameter_list;
+        if( p_actual->var_type == LIBVLC_STRING )
+        {
+            free( p_actual->default_value.psz_string );
+            free( p_actual->max_value.psz_string );
+            free( p_actual->min_value.psz_string );
+        }
+        free( p_actual->psz_parameter_name );
+        free( p_actual->psz_description );
+        p_parameter_list = p_actual->p_next;
+        free( p_actual );
+    }
+}
diff --git a/src/control/video.c b/src/control/video.c
index 61ba1af..66e7fe2 100644
--- a/src/control/video.c
+++ b/src/control/video.c
@@ -31,6 +31,33 @@
 #include <vlc_input.h>
 #include <vlc_vout.h>
 
+/* Names of filters
+ * it has to be written according to enum libvlc_video_filter_names_t
+ * in libvlc.h
+ */
+static const char * ppsz_filter_names[] = {
+    "adjust",      /* ADJUST */
+    "atmo",        /* ATMOLIGHT */
+    "bluescreen",  /* BLUESCREEN */
+    "colorthres",  /* COLORTHRESHOLD */
+    "deinterlace", /* DEINTERLACE */
+    "erase",       /* ERASE */
+    "extract",     /* EXTRACT */
+    "gaussianblur",/* GAUSSIAN_BLUR */
+    "gradient",    /* GRADIENT */
+    "grain",       /* GRAIN */
+    "invert",      /* INVERT */
+    "motionblur",  /* MOTIONBLUR */
+    "motiondetect",/* MOTIONDETECT */
+    "noise",       /* NOISE */
+    "postproc",    /* POSTPROCESS */
+    "psychedelic", /* PSYCHEDELIC */
+    "ripple",      /* RIPPLE */
+    "rotate",      /* ROTATE */
+    "sharpen",     /* SHARPEN */
+    "wave"         /* WAVE */
+};
+
 /*
  * Remember to release the returned vout_thread_t.
  */
@@ -280,6 +307,397 @@ void libvlc_video_set_size( libvlc_instance_t *p_instance, int width, int height
     }
 }
 
+/* declaration for callback */
+static void libvlc_video_filter_restore( libvlc_media_player_t *p_mediaplayer,
+                                         vout_thread_t *p_vout );
+
+/* is called when new vout is created, there is restore videofilter options */
+static int newVoutCallback( vlc_object_t *vlc_object, const char *variable, vlc_value_t old_val,
+                            vlc_value_t new_val, void *data )
+{
+    VLC_UNUSED( vlc_object );
+    VLC_UNUSED( variable );
+    VLC_UNUSED( old_val );
+    VLC_UNUSED( new_val );
+    libvlc_media_player_t *p_mediaplayer = ( libvlc_media_player_t * ) data;
+    libvlc_media_player_retain( p_mediaplayer );
+    libvlc_video_filter_restore( p_mediaplayer, ( vout_thread_t * ) new_val.p_object );
+    libvlc_media_player_release( p_mediaplayer );
+    return VLC_SUCCESS;
+}
+
+void libvlc_video_filter_add( libvlc_media_player_t *p_mediaplayer,
+                              libvlc_video_filter_names_t filter,
+                              libvlc_exception_t *p_e )
+{
+    static libvlc_media_player_t *p_callback_registered_player = NULL;
+    vout_thread_t *p_vout = GetVout( p_mediaplayer, p_e );
+
+    if( p_vout )
+    {
+        filter_chain_AppendFromString( p_vout->p_vf2_chain, ppsz_filter_names[filter] );
+
+        /* add callback for restoring filter options, when new video is played */
+        if( p_callback_registered_player != p_mediaplayer )
+        {
+            if( p_callback_registered_player )
+                var_DelCallback( p_vout->p_libvlc, "new-vout-filter",
+                    newVoutCallback, p_callback_registered_player );
+            var_AddCallback( p_vout->p_libvlc, "new-vout-filter", newVoutCallback, p_mediaplayer );
+            p_callback_registered_player = p_mediaplayer;
+        }
+        vlc_object_release( p_vout );
+
+        /* add filter to libvlc_filter_chain */
+        libvlc_filter_chain_t *p_actual = NULL;
+        libvlc_filter_chain_t *p_last = NULL;
+        if ( !p_mediaplayer->p_filter_chain )
+            p_actual = p_mediaplayer->p_filter_chain;
+        else
+        {
+            p_actual = p_mediaplayer->p_filter_chain->p_next_filter;
+            p_last = p_mediaplayer->p_filter_chain;
+            while( p_actual )
+            {
+                p_last = p_actual;
+                p_actual = p_actual->p_next_filter;
+            }
+        }
+
+        p_actual = ( libvlc_filter_chain_t * ) malloc( sizeof( libvlc_filter_chain_t ) );
+        if ( !p_actual )
+        {
+            libvlc_exception_raise( p_e, "Not enough memory" );
+            return;
+        }
+
+        /* fill with info */
+        p_actual->psz_filter = strdup( ppsz_filter_names[filter] );
+        p_actual->p_option = NULL;
+        p_actual->p_next_filter = NULL;
+
+        // connection
+        if ( p_last )
+            p_last->p_next_filter = p_actual;
+        else
+            p_mediaplayer->p_filter_chain = p_actual;
+    }
+}
+
+void libvlc_video_filter_remove( libvlc_media_player_t *p_mediaplayer,
+                                 libvlc_video_filter_names_t filter,
+                                 libvlc_exception_t *p_e )
+{
+    vout_thread_t *p_vout = GetVout( p_mediaplayer, p_e );
+
+    if( p_vout )
+    {
+        /* find filter to delete it with filter_t * */
+        filter_t *p_filter = filter_chain_GetFilter( p_vout->p_vf2_chain, 0, ppsz_filter_names[filter] );
+        filter_chain_DeleteFilter( p_vout->p_vf2_chain, p_filter );
+        vlc_object_release( p_vout );
+    }
+
+    /* remove filter from filter chain */
+    libvlc_filter_chain_t *p_actual = p_mediaplayer->p_filter_chain;
+    libvlc_filter_chain_t *p_before = p_actual;
+    while ( p_actual )
+    {
+        if ( strcmp( p_actual->psz_filter, ppsz_filter_names[filter] ) == 0 )
+        {
+            if ( p_before == p_actual )
+                p_mediaplayer->p_filter_chain = p_actual->p_next_filter;
+            else
+                p_before->p_next_filter = p_actual->p_next_filter;
+
+            /* free it */
+            while ( p_actual->p_option )
+            {
+                libvlc_filter_option_t * p_option = p_actual->p_option;
+                p_actual->p_option = p_option->p_next_option;
+                free( p_option->psz_option );
+                free( p_option );
+            }
+            free ( p_actual->psz_filter );
+            free ( p_actual );
+            break;
+        }
+        p_before = p_actual;
+        p_actual = p_actual->p_next_filter;
+    }
+}
+
+static void libvlc_video_filter_restore( libvlc_media_player_t *p_mediaplayer,
+                                         vout_thread_t *p_vout )
+{
+    if( p_vout && p_mediaplayer )
+    {
+        vlc_object_yield( p_vout );
+        libvlc_filter_chain_t* p_filter = p_mediaplayer->p_filter_chain;
+        while( p_filter )
+        {
+            filter_chain_AppendFromString( p_vout->p_vf2_chain, p_filter->psz_filter );
+            filter_t *p_f = filter_chain_GetFilter( p_vout->p_vf2_chain, 0, p_filter->psz_filter );
+            libvlc_filter_option_t *p_option = p_filter->p_option;
+            while ( p_option && p_f )
+            {
+                switch( p_option->type )
+                {
+                    case LIBVLC_BOOL:
+                        var_SetBool( p_f, p_option->psz_option, p_option->value.b_bool );
+                        break;
+                    case LIBVLC_INT:
+                        var_SetInteger( p_f, p_option->psz_option, p_option->value.i_int );
+                        break;
+                    case LIBVLC_FLOAT:
+                        var_SetFloat( p_f, p_option->psz_option, p_option->value.f_float );
+                        break;
+                    case LIBVLC_STRING:
+                        var_SetString( p_f, p_option->psz_option, p_option->value.psz_string );
+                        break;
+                    default:
+                        break;
+                }
+                p_option = p_option->p_next_option;
+            }
+            p_filter = p_filter->p_next_filter;
+        }
+        vlc_object_release( p_vout );
+    }
+}
+
+static void libvlc_video_filter_option_store( libvlc_media_player_t *p_mediaplayer,
+                                              const char *psz_filter,
+                                              const char *psz_option,
+                                              libvlc_var_type_t type,
+                                              libvlc_value_t value,
+                                              libvlc_exception_t *p_e )
+{
+    libvlc_filter_chain_t *p_actual = p_mediaplayer->p_filter_chain;
+    while ( p_actual )
+    {
+        if ( strcmp( p_actual->psz_filter, psz_filter ) == 0 )
+        {
+            libvlc_filter_option_t *p_option = p_actual->p_option;
+            libvlc_filter_option_t *p_before = p_option;
+
+            while ( p_option && strcmp( p_option->psz_option, psz_option ) != 0 )
+            {
+                p_before = p_option;
+                p_option = p_option->p_next_option;
+            }
+
+            if ( !p_option)
+            {
+                p_option = ( libvlc_filter_option_t * )
+                    malloc( sizeof( libvlc_filter_option_t ) );
+                if ( !p_option )
+                {
+                    libvlc_exception_raise( p_e, "Not enough memory" );
+                    return;
+                }
+                if ( p_before == NULL )  /* first write */
+                    p_actual->p_option = p_option;
+                else
+                    p_before->p_next_option = p_option;
+                p_option->psz_option = strdup( psz_option );
+                p_option->p_next_option = NULL;
+            }
+            /* store info */
+            p_option->value = value;
+            p_option->type = type;
+            break;
+        }
+        p_actual = p_actual->p_next_filter;
+    }
+}
+
+/**
+ * Get info about video filter parameters
+ */
+libvlc_filter_parameter_list_t *libvlc_video_filter_get_parameters(
+        libvlc_instance_t *p_instance,
+        libvlc_video_filter_names_t filter,
+        libvlc_exception_t *p_e )
+{
+    module_t * p_module = module_Find( p_instance->p_libvlc_int, ppsz_filter_names[filter] );
+    if( !p_module )
+    {
+        libvlc_exception_raise( p_e, "Unknown module" );
+        return NULL;
+    }
+    libvlc_filter_parameter_list_t *p_list = libvlc_filter_get_parameters( p_module, p_e );
+    module_Put( p_module );
+
+    return p_list;
+}
+
+/**
+ * Set parameter for video filter
+ */
+void libvlc_video_filter_set_parameter(
+        libvlc_media_player_t *p_mediaplayer,
+        libvlc_video_filter_names_t filter,
+        const char *psz_parameter_name,
+        libvlc_var_type_t type,
+        libvlc_value_t new_value,
+        libvlc_exception_t *p_e )
+{
+    libvlc_filter_set_parameter(
+        p_mediaplayer->p_libvlc_instance,
+        psz_parameter_name,
+        type,
+        new_value );
+    /* try apply it directly */
+    vout_thread_t *p_vout = GetVout( p_mediaplayer, p_e );
+
+    if( p_vout )
+    {
+        filter_t *p_filter = filter_chain_GetFilter( p_vout->p_vf2_chain, 0, ppsz_filter_names[filter] );
+
+        if ( p_filter )
+        {
+            switch( type )
+            {
+                case LIBVLC_BOOL:
+                    var_SetBool( p_filter, psz_parameter_name, new_value.b_bool );
+                    break;
+                case LIBVLC_INT:
+                    var_SetInteger( p_filter, psz_parameter_name, new_value.i_int );
+                    break;
+                case LIBVLC_FLOAT:
+                    var_SetFloat( p_filter, psz_parameter_name, new_value.f_float );
+                    break;
+                case LIBVLC_STRING:
+                    var_SetString( p_filter, psz_parameter_name, new_value.psz_string );
+                    break;
+                default:
+                    break;
+            }
+            libvlc_video_filter_option_store( p_mediaplayer, ppsz_filter_names[filter],
+                                              psz_parameter_name, type, new_value, p_e );
+        }
+
+        vlc_object_release( p_vout );
+    }
+}
+
+void libvlc_video_set_brightness( libvlc_media_player_t *p_mediaplayer,
+                                  float f_brightness,
+                                  libvlc_exception_t *p_e )
+{
+    vout_thread_t *p_vout = GetVout( p_mediaplayer, p_e );
+
+    if( p_vout )
+    {
+        filter_t *p_filter = filter_chain_GetFilter( p_vout->p_vf2_chain, 0, ppsz_filter_names[ADJUST] );
+
+        if ( p_filter )
+        {
+            var_SetFloat( p_filter, "brightness", f_brightness );
+            libvlc_value_t val;
+            val.f_float = f_brightness;
+            libvlc_video_filter_option_store( p_mediaplayer, ppsz_filter_names[ADJUST],
+                                              "brightness", LIBVLC_FLOAT, val, p_e );
+        }
+
+        vlc_object_release( p_vout );
+    }
+}
+
+void libvlc_video_set_contrast( libvlc_media_player_t *p_mediaplayer,
+                                float f_contrast,
+                                libvlc_exception_t *p_e )
+{
+    vout_thread_t *p_vout = GetVout( p_mediaplayer, p_e );
+
+    if( p_vout )
+    {
+        filter_t *p_filter = filter_chain_GetFilter( p_vout->p_vf2_chain, 0, ppsz_filter_names[ADJUST] );
+
+        if ( p_filter )
+        {
+            var_SetFloat( p_filter, "contrast", f_contrast );
+            libvlc_value_t val;
+            val.f_float = f_contrast;
+            libvlc_video_filter_option_store( p_mediaplayer, ppsz_filter_names[ADJUST],
+                                              "contrast", LIBVLC_FLOAT, val, p_e );
+        }
+
+        vlc_object_release( p_vout );
+    }
+}
+
+void libvlc_video_set_gamma( libvlc_media_player_t *p_mediaplayer,
+                             float f_gamma,
+                             libvlc_exception_t *p_e )
+{
+
+    vout_thread_t *p_vout = GetVout( p_mediaplayer, p_e );
+
+    if( p_vout )
+    {
+        filter_t *p_filter = filter_chain_GetFilter( p_vout->p_vf2_chain, 0, ppsz_filter_names[ADJUST] );
+
+        if ( p_filter )
+        {
+            var_SetFloat( p_filter, "gamma", f_gamma );
+            libvlc_value_t val;
+            val.f_float = f_gamma;
+            libvlc_video_filter_option_store( p_mediaplayer, ppsz_filter_names[ADJUST],
+                                              "gamma", LIBVLC_FLOAT, val, p_e );
+        }
+
+        vlc_object_release( p_vout );
+    }
+}
+
+void libvlc_video_set_hue( libvlc_media_player_t *p_mediaplayer,
+                           int i_hue,
+                           libvlc_exception_t *p_e )
+{
+    vout_thread_t *p_vout = GetVout( p_mediaplayer, p_e );
+
+    if( p_vout )
+    {
+        filter_t *p_filter = filter_chain_GetFilter( p_vout->p_vf2_chain, 0, ppsz_filter_names[ADJUST] );
+
+        if ( p_filter )
+        {
+            var_SetInteger( p_filter, "hue", i_hue );
+            libvlc_value_t val;
+            val.i_int = i_hue;
+            libvlc_video_filter_option_store( p_mediaplayer, ppsz_filter_names[ADJUST],
+                                              "hue", LIBVLC_INT, val, p_e );
+        }
+
+        vlc_object_release( p_vout );
+    }
+}
+
+void libvlc_video_set_saturation( libvlc_media_player_t *p_mediaplayer,
+                                  float f_saturation,
+                                  libvlc_exception_t *p_e )
+{
+    vout_thread_t *p_vout = GetVout( p_mediaplayer, p_e );
+
+    if( p_vout )
+    {
+        filter_t *p_filter = filter_chain_GetFilter( p_vout->p_vf2_chain, 0, ppsz_filter_names[ADJUST] );
+
+        if ( p_filter )
+        {
+            var_SetFloat( p_filter, "saturation", f_saturation );
+            libvlc_value_t val;
+            val.f_float = f_saturation;
+            libvlc_video_filter_option_store( p_mediaplayer, ppsz_filter_names[ADJUST],
+                                              "saturation", LIBVLC_FLOAT, val, p_e );
+        }
+
+        vlc_object_release( p_vout );
+    }
+}
+
 void libvlc_video_set_viewport( libvlc_instance_t *p_instance,
                             const libvlc_rectangle_t *view, const libvlc_rectangle_t *clip,
                            libvlc_exception_t *p_e )
@@ -383,6 +801,27 @@ int libvlc_video_get_spu( libvlc_media_player_t *p_mi,
     return i_spu;
 }
 
+int libvlc_video_get_spu_count( libvlc_media_player_t *p_mi,
+                                  libvlc_exception_t *p_e )
+{
+    input_thread_t *p_input_thread = libvlc_get_input_thread( p_mi, p_e );
+    vlc_value_t val_list;
+
+    if( !p_input_thread )
+        return -1;
+
+    var_Change( p_input_thread, "spu-es", VLC_VAR_GETCHOICES, &val_list, NULL );
+    vlc_object_release( p_input_thread );
+    return val_list.p_list->i_count;
+}
+
+libvlc_track_description_t *
+        libvlc_video_get_spu_description( libvlc_media_player_t *p_mi,
+                                          libvlc_exception_t *p_e )
+{
+    return libvlc_get_track_description( p_mi, "spu-es", p_e);
+}
+
 void libvlc_video_set_spu( libvlc_media_player_t *p_mi, int i_spu,
                            libvlc_exception_t *p_e )
 {
@@ -431,6 +870,23 @@ int libvlc_video_set_subtitle_file( libvlc_media_player_t *p_mi,
     return b_ret;
 }
 
+libvlc_track_description_t *
+        libvlc_video_get_title_description( libvlc_media_player_t *p_mi,
+                                            libvlc_exception_t * p_e )
+{
+    return libvlc_get_track_description( p_mi, "title", p_e);
+}
+
+libvlc_track_description_t *
+        libvlc_video_get_chapter_description( libvlc_media_player_t *p_mi,
+                                              int i_title,
+                                              libvlc_exception_t *p_e )
+{
+    char psz_title[12];
+    sprintf( psz_title,  "title %2i", i_title );
+    return libvlc_get_track_description( p_mi, psz_title, p_e);
+}
+
 char *libvlc_video_get_crop_geometry( libvlc_media_player_t *p_mi,
                                    libvlc_exception_t *p_e )
 {
@@ -522,6 +978,92 @@ void libvlc_toggle_teletext( libvlc_media_player_t *p_mi,
     vlc_object_release( p_vout );
 }
 
+int libvlc_video_get_track_count( libvlc_media_player_t *p_mi,
+                                  libvlc_exception_t *p_e )
+{
+    input_thread_t *p_input_thread = libvlc_get_input_thread( p_mi, p_e );
+    vlc_value_t val_list;
+
+    if( !p_input_thread )
+        return -1;
+
+    var_Change( p_input_thread, "video-es", VLC_VAR_GETCHOICES, &val_list, NULL );
+    vlc_object_release( p_input_thread );
+    return val_list.p_list->i_count;
+}
+
+libvlc_track_description_t *
+        libvlc_video_get_track_description( libvlc_media_player_t *p_mi,
+                                            libvlc_exception_t *p_e )
+{
+    return libvlc_get_track_description( p_mi, "video-es", p_e);
+}
+
+int libvlc_video_get_track( libvlc_media_player_t *p_mi,
+                            libvlc_exception_t *p_e )
+{
+    input_thread_t *p_input_thread = libvlc_get_input_thread( p_mi, p_e );
+    vlc_value_t val_list;
+    vlc_value_t val;
+    int i_track = -1;
+    int i_ret = -1;
+    int i;
+
+    if( !p_input_thread )
+        return -1;
+
+    i_ret = var_Get( p_input_thread, "video-es", &val );
+    if( i_ret < 0 )
+    {
+        libvlc_exception_raise( p_e, "Getting Video track information failed" );
+        vlc_object_release( p_input_thread );
+        return i_ret;
+    }
+
+    var_Change( p_input_thread, "video-es", VLC_VAR_GETCHOICES, &val_list, NULL );
+    for( i = 0; i < val_list.p_list->i_count; i++ )
+    {
+        vlc_value_t track_val = val_list.p_list->p_values[i];
+        if( track_val.i_int == val.i_int )
+        {
+            i_track = i;
+            break;
+       }
+    }
+    vlc_object_release( p_input_thread );
+    return i_track;
+}
+
+void libvlc_video_set_track( libvlc_media_player_t *p_mi, int i_track,
+                             libvlc_exception_t *p_e )
+{
+    input_thread_t *p_input_thread = libvlc_get_input_thread( p_mi, p_e );
+    vlc_value_t val_list;
+    int i_ret = -1;
+    int i;
+
+    if( !p_input_thread )
+        return;
+
+    var_Change( p_input_thread, "video-es", VLC_VAR_GETCHOICES, &val_list, NULL );
+    for( i = 0; i < val_list.p_list->i_count; i++ )
+    {
+        vlc_value_t val = val_list.p_list->p_values[i];
+        if( i_track == val.i_int )
+        {
+            i_ret = var_Set( p_input_thread, "audio-es", val );
+            if( i_ret < 0 )
+            {
+                libvlc_exception_raise( p_e, "Setting video track failed" );
+            }
+            vlc_object_release( p_input_thread );
+            return;
+        }
+    }
+    libvlc_exception_raise( p_e, "Video track out of range" );
+    vlc_object_release( p_input_thread );
+}
+
 int libvlc_video_destroy( libvlc_media_player_t *p_mi,
                           libvlc_exception_t *p_e )
 {
diff --git a/src/libvlc.c b/src/libvlc.c
index 96cab09..0ea0451 100644
--- a/src/libvlc.c
+++ b/src/libvlc.c
@@ -938,6 +938,10 @@ int libvlc_InternalInit( libvlc_int_t *p_libvlc, int i_argc,
 
     var_Create( p_libvlc, "intf-popupmenu", VLC_VAR_BOOL );
 
+    /* For having info, that new video output was created and video
+       filter chain initialized*/
+    var_Create( p_libvlc, "new-vout-filter", VLC_VAR_ADDRESS );
+
     /*
      * Get input filenames given as commandline arguments
      */
diff --git a/src/libvlc.sym b/src/libvlc.sym
index ce03edd..0945020 100644
--- a/src/libvlc.sym
+++ b/src/libvlc.sym
@@ -1,10 +1,24 @@
 libvlc_add_intf
+libvlc_audio_alsa_device_count
+libvlc_audio_alsa_device_id
+libvlc_audio_alsa_device_longname
+libvlc_audio_alsa_device_set
+libvlc_audio_device_available
+libvlc_audio_device_set
+libvlc_audio_equalizer_set_preset
+libvlc_audio_filter_add
+libvlc_audio_filter_get_parameters
+libvlc_audio_filter_remove
+libvlc_audio_filter_set_parameter
 libvlc_audio_get_channel
+libvlc_audio_get_device
 libvlc_audio_get_mute
 libvlc_audio_get_track
 libvlc_audio_get_track_count
+libvlc_audio_get_track_description
 libvlc_audio_get_volume
 libvlc_audio_set_channel
+libvlc_audio_set_device
 libvlc_audio_set_mute
 libvlc_audio_set_track
 libvlc_audio_set_volume
@@ -21,6 +35,7 @@ libvlc_exception_get_message
 libvlc_exception_init
 libvlc_exception_raise
 libvlc_exception_raised
+libvlc_filter_parameters_release
 libvlc_get_changeset
 libvlc_get_compiler
 libvlc_get_fullscreen
@@ -110,6 +125,7 @@ libvlc_media_player_destroy
 libvlc_media_player_event_manager
 libvlc_media_player_get_chapter
 libvlc_media_player_get_chapter_count
+libvlc_media_player_get_chapter_count_for_title
 libvlc_media_player_get_drawable
 libvlc_media_player_get_fps
 libvlc_media_player_get_length
@@ -118,13 +134,17 @@ libvlc_media_player_get_position
 libvlc_media_player_get_rate
 libvlc_media_player_get_state
 libvlc_media_player_get_time
+libvlc_media_player_get_title
+libvlc_media_player_get_title_count
 libvlc_media_player_has_vout
 libvlc_media_player_is_seekable
 libvlc_media_player_new
 libvlc_media_player_new_from_input_thread
 libvlc_media_player_new_from_media
+libvlc_media_player_next_chapter
 libvlc_media_player_pause
 libvlc_media_player_play
+libvlc_media_player_previous_chapter
 libvlc_media_player_release
 libvlc_media_player_retain
 libvlc_media_player_set_chapter
@@ -133,6 +153,7 @@ libvlc_media_player_set_media
 libvlc_media_player_set_position
 libvlc_media_player_set_rate
 libvlc_media_player_set_time
+libvlc_media_player_set_title
 libvlc_media_player_stop
 libvlc_media_player_will_play
 libvlc_media_release
@@ -163,24 +184,42 @@ libvlc_set_fullscreen
 libvlc_set_log_verbosity
 libvlc_toggle_fullscreen
 libvlc_toggle_teletext
+libvlc_track_description_release
 libvlc_video_destroy
+libvlc_video_filter_add
+libvlc_video_filter_get_parameters
+libvlc_video_filter_remove
+libvlc_video_filter_set_parameter
 libvlc_video_get_aspect_ratio
+libvlc_video_get_chapter_description
 libvlc_video_get_crop_geometry
 libvlc_video_get_height
 libvlc_video_get_parent
 libvlc_video_get_spu
+libvlc_video_get_spu_count
+libvlc_video_get_spu_description
 libvlc_video_get_teletext
+libvlc_video_get_title_description
+libvlc_video_get_track
+libvlc_video_get_track_count
+libvlc_video_get_track_description
 libvlc_video_get_width
 libvlc_video_redraw_rectangle
 libvlc_video_reparent
 libvlc_video_resize
 libvlc_video_set_aspect_ratio
+libvlc_video_set_brightness
+libvlc_video_set_contrast
 libvlc_video_set_crop_geometry
+libvlc_video_set_gamma
+libvlc_video_set_hue
 libvlc_video_set_parent
+libvlc_video_set_saturation
 libvlc_video_set_size
 libvlc_video_set_spu
 libvlc_video_set_subtitle_file
 libvlc_video_set_teletext
+libvlc_video_set_track
 libvlc_video_set_viewport
 libvlc_video_take_snapshot
 libvlc_vlm_add_broadcast
diff --git a/src/video_output/video_output.c b/src/video_output/video_output.c
index 6be98a6..690d646 100644
--- a/src/video_output/video_output.c
+++ b/src/video_output/video_output.c
@@ -1101,6 +1101,10 @@ static void* RunThread( vlc_object_t *p_this )
 
             free( p_vout->psz_vf2 );
             p_vout->psz_vf2 = NULL;
+            /* Write adrress to indicate new vout was created */
+            vlc_value_t value;
+            value.p_object = ( vlc_object_t * ) p_vout;
+            var_Set( p_vout->p_libvlc, "new-vout-filter", value );
         }
         vlc_mutex_unlock( &p_vout->vfilter_lock );
     }
-- 
1.5.5.1

